begin
  require 'fortran'
rescue LoadError
  STDERR.puts "gem install fortran"
  exit 1
end

require 'funit/compiler'
require 'funit/functions'
require 'funit/assertions'
require 'funit/testsuite'
require 'funit/c_tools'
require 'fileutils'

module Funit

  VERSION = '0.12.4'

  ##
  # run all tests

  def run_tests(funit_data)
    Compiler.new# a test for compiler env set (FIXME: remove this later)
    write_test_runner( test_files = parse_command_line )
    test_suites = []
    test_files.each{ |test_file|
      tf_content = IO.read(test_file+'.fun')
      tf_content.scan(/test_suite\s+(\w+)(.*?)end\s+test_suite/m).each{|ts|
        ts_name = $1
        ts_content = $2
        if((!File.exist?(ts_name+"_fun.f90")) || File.mtime(ts_name+"_fun.f90") < File.mtime(test_file+".fun")) then
          source_file = Dir.glob("#{ts_name}.*90").first
          if ( File.read(source_file).match(/\s*module\s+#{ts_name}/i) ) then
            TestSuite.new(ts_name, ts_content, false)
          else
            TestSuite.new(ts_name, ts_content, true)
          end
        end
        test_suites.push(ts_name)
      }
    }
    compile_tests(test_suites,funit_data)
    exit 1 unless system "env PATH='.' TestRunner"
  end

  ##
  # remove files generated by fUnit

  def clean_genFiles
    module_names = Dir["**/*.fun"].map{|mn| mn.chomp(".fun")}
    tbCancelled = module_names.map{|mn| mn+"_fun."} + ["TestRunner."]
    tbCancelled = tbCancelled.map{|tbc| [tbc+"f90",tbc+"o",tbc+"MOD"]}.flatten
    tbCancelled += Dir["**/TestRunner"]
    tbCancelled += Dir["**/makeTestRunner"]
    tbCancelled = (tbCancelled+tbCancelled.map{|tbc| tbc.downcase}).uniq
    FileUtils.rm_f(tbCancelled)
  end

  def check_for_FSFLAG
    if(ENV['FSFLAG'].nil?) then
      puts <<-EOF
No environment variable FSFLAG set.

For example for most compilers such as gfortran you will need: -I
      sh: export FSFLAG=-I
     csh: setenv FSFLAG -I
 windows: set FSFLAG=-I

but for some such as Sun's f95 compiler you will need: -M
      sh: export FSFLAG=-M
     csh: setenv FSFLAG -M
 windows: set FSFLAG=-M
    EOF
    exit
    end
  end

  def print_env_flags
    puts <<-EOF

Some influential environment variables:

  FC            fortran compiler command
  FCFLAGS       fortran compiler flags
  LDFLAGS       linker flags

  CC            C compiler command
  CFLAGS        C compiler flags

  CXX           C++ compiler command
  CXXFLAGS      C++ compiler flags

  MAKE_OPTS     make commandline options (e.g. "-j")

    EOF
  end
end

#--
# Copyright 2006-2007 United States Government as represented by
# NASA Langley Research Center. No copyright is claimed in
# the United States under Title 17, U.S. Code. All Other Rights
# Reserved.
#
# This file is governed by the NASA Open Source Agreement.
# See License.txt for details.
#++
